# mnist_dnn.py
# Keras 2.1.4 TensorFlow 1.4.0
# Anaconda3 4.1.1 (Python 3.5.2)

import numpy as np
import keras as K

import os
os.environ['TF_CPP_MIN_LOG_LEVEL']='2'

def main():
  print("\nMNIST DNN demo using Keras/TensorFlow ")

  print("Loading MNIST training data into memory \n")
  train_file = ".\\Data\\mnist_train_keras_1000.txt"
  train_x = np.loadtxt(train_file, usecols=range(11,795),
    delimiter=" ",  skiprows=0, dtype=np.float32)
  train_y = np.loadtxt(train_file, usecols=range(0,10),
    delimiter=" ", skiprows=0, dtype=np.float32)

  train_x /= 255  # normalize

  print("Creating a 784-(400-400)-10 deep NN \n")
  np.random.seed(1)
  model = K.models.Sequential()
  model.add(K.layers.Dense(units=400, input_dim=784,
    activation='relu'))
  model.add(K.layers.Dense(units=400,
    activation='relu'))
  model.add(K.layers.Dense(units=10, activation='softmax'))
  model.compile(loss='categorical_crossentropy',
    optimizer='rmsprop', metrics=['accuracy'])

  print("Starting training ")
  num_epochs = 10
  h = model.fit(train_x, train_y, batch_size=50,
    epochs=num_epochs, verbose=0)  # 0=none, 1=chatty, 2=med
  print("Training finished \n")

  # print(h.history.keys())  # dict_keys(['acc', 'loss'])

  print("Training history: ")
  for i in range(num_epochs):
    if i % 1 == 0:
      los = h.history['loss'][i]
      acc = h.history['acc'][i] * 100
      print("epoch: %5d loss = %0.4f acc = %0.2f%%" \
        % (i, los, acc))

  test_file = ".\\Data\\mnist_test_keras_100.txt"
  test_x = np.loadtxt(test_file, usecols=range(11,795),
    delimiter=" ",  skiprows=0, dtype=np.float32)
  test_y = np.loadtxt(test_file, usecols=range(0,10),
    delimiter=" ", skiprows=0, dtype=np.float32)  

  eval = model.evaluate(test_x, test_y, verbose=0)
  print("\nEvaluation on test data: \nloss = %0.4f \
    accuracy = %0.2f%%" % (eval[0], eval[1]*100) )

  

  # pixels_list = [0.55] * 784
  # pixels_vec = np.array(pixels_list, dtype=np.float32)
  # pixels_mat = pixels_vec.reshape((1,784))
  # pred_probs = model.predict(pixels_mat)
  # pred_digit = np.argmax(pred_probs)
  # print(pred_digit)

  print("\nEnd MNIST DNN demo \n")

if __name__=="__main__":
  main()
